package eu.geclipse.jobsubmission;

import java.io.File;
import java.net.URI;
import java.util.List;

import org.eclipse.core.internal.net.ProxyData;
import org.eclipse.core.net.proxy.IProxyData;
import org.eclipse.core.net.proxy.IProxyService;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.IScopeContext;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.prefs.BackingStoreException;

import eu.geclipse.core.ExtensionManager;
import eu.geclipse.core.Extensions;
import eu.geclipse.core.auth.AuthenticationException;
import eu.geclipse.core.auth.AuthenticationTokenManager;
import eu.geclipse.core.auth.CaCertManager;
import eu.geclipse.core.auth.IAuthenticationToken;
import eu.geclipse.core.auth.ICaCertificate;
import eu.geclipse.core.auth.ICaCertificateLoader;
import eu.geclipse.core.auth.PasswordManager;
import eu.geclipse.core.model.GridModel;
import eu.geclipse.core.model.GridModelException;
import eu.geclipse.core.model.IGridJobDescription;
import eu.geclipse.core.model.IGridJobID;
import eu.geclipse.core.model.IGridJobService;
import eu.geclipse.core.model.IVirtualOrganization;
import eu.geclipse.core.model.IVoManager;
import eu.geclipse.core.project.GridProjectNature;
import eu.geclipse.core.reporting.ProblemException;
import eu.geclipse.jobsubmission.internal.Activator;
import eu.geclipse.jsdl.JSDLJobDescription;
import eu.geclipse.voms.auth.VomsProxyDescription;
import eu.geclipse.voms.vo.VomsVoCreator;

/**
 * This class controls all aspects of the application's execution
 */
public class Application implements IApplication {
  
  private static final String PROJECT_NAME = your_project_name;
  
  private static final String VO_NAME = your_vo_name;
  
  private static final String VO_HOST = your_vo_host;
  
  private static final int VO_PORT = your_vo_port;
  
  private static final String VO_HOST_DN = your_vo_host_dn;
  
  private static final String VO_INFO_SERVICE = your_vo_info_service;
  
  private static final File CERT_FILE = new File( your_cert_file );
  
  private static final File KEY_FILE = new File( your_key_file );
  
  private static final String JOB_DESCRIPTION_FILE = your_sample_jsdl_file;
  
  private static final String PW = your_password;

	/* (non-Javadoc)
	 * @see org.eclipse.equinox.app.IApplication#start(org.eclipse.equinox.app.IApplicationContext)
	 */
	public Object start( final IApplicationContext context ) throws Exception {
	  
	  PasswordManager.registerPassword( KEY_FILE.getPath(), PW );
	  
	  // Only needed if you are behind a proxy
	  // setupProxy();
	  
	  try {
	    
  	  System.out.println( "Creating VO..." );
  	  IVirtualOrganization vo = createVO();
  	  
  	  System.out.println( "Fetching job submission services..." );
  	  IGridJobService[] jss = vo.getJobSubmissionServices( null );
  	  if ( ( jss == null ) || ( jss.length == 0 ) ) {
  	    System.err.println( "No job submission services found for VO: " + vo.getName() );
  	    return IApplication.EXIT_OK;
  	  }
  	  
  	  System.out.println( "Creating project..." );
  	  IProject project = createProject( vo );
  	  
  	  System.out.println( "Creating job description" );
  	  IGridJobDescription description = createJobDescription( project );
  	  
  	  System.out.println( "Importing CAs..." );
  	  importCAs();
  	  
  	  System.out.println( "Creating authentication token..." );
      createAuthToken( vo );
  	  
      System.out.println( "Submitting job to " + jss[ 0 ].getName() );
  	  IGridJobID jobID = jss[ 0 ].submitJob( description, null );
  	  
  	  System.out.println( "Job submission finished. Job ID is: " + jobID.getJobID() );
  	  
	  } catch ( Exception exc ) {
	    exc.printStackTrace();
	  }
  	  
	  return IApplication.EXIT_OK;
		
	}
	
	/* (non-Javadoc)
   * @see org.eclipse.equinox.app.IApplication#stop()
   */
  public void stop() {
    // nothing to do
  }
	
	protected IAuthenticationToken createAuthToken( final IVirtualOrganization vo ) throws AuthenticationException {
    
	  AuthenticationTokenManager manager = AuthenticationTokenManager.getManager();
	  
    VomsProxyDescription desc = new VomsProxyDescription();
    desc.setVo( vo );
    desc.setCertFile( CERT_FILE );
    desc.setKeyFile( KEY_FILE );
    desc.setLifetime( 86400 );
    
    IAuthenticationToken token = manager.createToken( desc );
    token.validate();
    token.setActive( true );
    
    return token;
    
  }
	
	protected IGridJobDescription createJobDescription( final IContainer parent ) throws CoreException {
	  
	  IFile file = parent.getFile( new Path( "myjsdl.jsdl" ) ); //$NON-NLS-1$
	  if ( ! file.exists() ) {
	    file.createLink( new Path( JOB_DESCRIPTION_FILE ), IResource.NONE, null );
	  }

	  return new JSDLJobDescription( file );
	  
	}
		
	protected IProject createProject( final IVirtualOrganization vo ) throws CoreException, BackingStoreException {
	  
	  IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
    IProject project = workspaceRoot.getProject( PROJECT_NAME );
    
    if ( ! project.exists() ) {
      project.create( null );
    }
    
    project.open( null );
    
    IScopeContext projectScope = new ProjectScope( project );
    IEclipsePreferences projectNode = projectScope.getNode( "eu.geclipse.core" ); //$NON-NLS-1$
    projectNode.put( "vo", vo.getName() ); //$NON-NLS-1$
    projectNode.flush();
    
    project.getDescription().setNatureIds( new String[] { GridProjectNature.getID() } );
    
    return project;
    
	}
	
	protected IVirtualOrganization createVO() throws GridModelException {
	  
	  IVoManager manager = GridModel.getVoManager();
	  
	  VomsVoCreator creator = new VomsVoCreator();
	  creator.setVoName( VO_NAME );
	  creator.setVoHost( VO_HOST );
	  creator.setVoPort( VO_PORT );
	  creator.setVoHostDN( VO_HOST_DN );
	  creator.setVoInfoService( URI.create( VO_INFO_SERVICE ) );
	  
	  return ( IVirtualOrganization ) manager.create( creator );
	  
	}
	
	protected void importCAs() throws ProblemException {
	  
	  CaCertManager manager = CaCertManager.getManager();
	  
	  ExtensionManager extManager = new ExtensionManager();
	  List< Object > loaders
	    = extManager.getExecutableExtensions(
	        Extensions.CA_CERT_LOADER_POINT,
	        Extensions.CA_CERT_LOADER_ELEMENT,
	        Extensions.CA_CERT_LOADER_CLASS_ATTRIBUTE
	    );
	  
	  if ( loaders != null ) {
	    for ( Object loader : loaders ) {
	      ICaCertificateLoader caLoader = ( ICaCertificateLoader ) loader;
	      URI[] locations = caLoader.getPredefinedRemoteLocations();
	      if ( locations != null ) {
	        for ( URI uri : locations ) {
	          String[] certificateList = caLoader.getCertificateList( uri, null );
	          for ( String certID : certificateList ) {
	            ICaCertificate cert = caLoader.getCertificate( uri, certID, null );
	            manager.addCertificate( cert );
	          }
	        }
	      }
	    }
	  }
	  
	}
	
	protected void setupProxy() throws CoreException {
	  
	  BundleContext context = Activator.getDefault().getBundle().getBundleContext();
	  ServiceReference reference = context.getServiceReference( IProxyService.class.getName() );
	  IProxyService service = ( IProxyService ) context.getService( reference );
	  
	  service.setProxiesEnabled( true );
	  service.setProxyData(
	      new IProxyData[] {
	        new ProxyData( IProxyData.HTTP_PROXY_TYPE, your_proxy_host, your_proxy_port, false),
	        new ProxyData( IProxyData.HTTPS_PROXY_TYPE, your_proxy_host, your_proxy_port, false)
	      }
	  );
	  
	}
	
}
